@@ -94,7 +94,7 @@ struct _TumblerCacheService
   GThreadPool     *delete_pool;
   GThreadPool     *cleanup_pool;
 
-  GMutex          *mutex;
+  TUMBLER_MUTEX    (mutex);
 };
 
 struct _MoveRequest
@@ -150,7 +150,7 @@ tumbler_cache_service_class_init (TumblerCacheServiceClass *klass)
 static void
 tumbler_cache_service_init (TumblerCacheService *service)
 {
-  service->mutex = g_mutex_new ();
+  tumbler_mutex_create (service->mutex);
 }
 
 
@@ -174,6 +174,15 @@ tumbler_cache_service_constructed (GObject *object)
                                             service, 1, FALSE, NULL);
   service->cleanup_pool = g_thread_pool_new (tumbler_cache_service_cleanup_thread, 
                                              service, 1, FALSE, NULL);
+
+  /* everything's fine, install the cache type D-Bus info */
+  dbus_g_object_type_install_info (G_OBJECT_TYPE (service),
+                                   &dbus_glib_tumbler_cache_service_object_info);
+
+  /* register the cache instance as a handler of the cache interface */
+  dbus_g_connection_register_g_object (service->connection, 
+                                       "/org/freedesktop/thumbnails/Cache1",
+                                       G_OBJECT (service));
 }
 
 
@@ -193,7 +202,7 @@ tumbler_cache_service_finalize (GObject *object)
 
   dbus_g_connection_unref (service->connection);
 
-  g_mutex_free (service->mutex);
+  tumbler_mutex_free (service->mutex);
 
   (*G_OBJECT_CLASS (tumbler_cache_service_parent_class)->finalize) (object);
 }
@@ -252,7 +261,7 @@ tumbler_cache_service_move_thread (gpointer data,
   g_return_if_fail (TUMBLER_IS_CACHE_SERVICE (service));
   g_return_if_fail (request != NULL);
 
-  g_mutex_lock (service->mutex);
+  tumbler_mutex_lock (service->mutex);
 
   if (service->cache != NULL)
     {
@@ -269,7 +278,7 @@ tumbler_cache_service_move_thread (gpointer data,
    * other requests are still to be processed) */
   tumbler_component_decrement_use_count (TUMBLER_COMPONENT (service));
 
-  g_mutex_unlock (service->mutex);
+  tumbler_mutex_unlock (service->mutex);
 }
 
 
@@ -284,7 +293,7 @@ tumbler_cache_service_copy_thread (gpointer data,
   g_return_if_fail (TUMBLER_IS_CACHE_SERVICE (service));
   g_return_if_fail (request != NULL);
 
-  g_mutex_lock (service->mutex);
+  tumbler_mutex_lock (service->mutex);
 
   if (service->cache != NULL)
     {
@@ -301,7 +310,7 @@ tumbler_cache_service_copy_thread (gpointer data,
    * other requests are still to be processed) */
   tumbler_component_decrement_use_count (TUMBLER_COMPONENT (service));
 
-  g_mutex_unlock (service->mutex);
+  tumbler_mutex_unlock (service->mutex);
 }
 
 
@@ -316,7 +325,7 @@ tumbler_cache_service_delete_thread (gpointer data,
   g_return_if_fail (TUMBLER_IS_CACHE_SERVICE (service));
   g_return_if_fail (request != NULL);
 
-  g_mutex_lock (service->mutex);
+  tumbler_mutex_lock (service->mutex);
 
   if (service->cache != NULL)
     tumbler_cache_delete (service->cache, (const gchar *const *)request->uris);
@@ -328,7 +337,7 @@ tumbler_cache_service_delete_thread (gpointer data,
    * other requests are still to be processed) */
   tumbler_component_decrement_use_count (TUMBLER_COMPONENT (service));
 
-  g_mutex_unlock (service->mutex);
+  tumbler_mutex_unlock (service->mutex);
 }
 
 
@@ -343,7 +352,7 @@ tumbler_cache_service_cleanup_thread (gpointer data,
   g_return_if_fail (TUMBLER_IS_CACHE_SERVICE (service));
   g_return_if_fail (request != NULL);
 
-  g_mutex_lock (service->mutex);
+  tumbler_mutex_lock (service->mutex);
 
   if (service->cache != NULL)
     {
@@ -359,7 +368,7 @@ tumbler_cache_service_cleanup_thread (gpointer data,
    * other requests are still to be processed) */
   tumbler_component_decrement_use_count (TUMBLER_COMPONENT (service));
 
-  g_mutex_unlock (service->mutex);
+  tumbler_mutex_unlock (service->mutex);
 }
 
 
@@ -381,56 +390,35 @@ tumbler_cache_service_start (TumblerCacheService *service,
                              GError             **error)
 {
   DBusConnection *connection;
-  DBusError       dbus_error;
   gint            result;
 
   g_return_val_if_fail (TUMBLER_IS_CACHE_SERVICE (service), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  g_mutex_lock (service->mutex);
-
-  /* initialize the D-Bus error */
-  dbus_error_init (&dbus_error);
+  tumbler_mutex_lock (service->mutex);
 
   /* get the native D-Bus connection */
   connection = dbus_g_connection_get_connection (service->connection);
 
   /* request ownership for the cache interface */
   result = dbus_bus_request_name (connection, "org.freedesktop.thumbnails.Cache1", 
-                                  DBUS_NAME_FLAG_DO_NOT_QUEUE, &dbus_error);
+                                  DBUS_NAME_FLAG_DO_NOT_QUEUE, NULL);
 
   /* check if that failed */
   if (result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
     {
-      /* propagate the D-Bus error */
-      if (dbus_error_is_set (&dbus_error))
-        {
-          if (error != NULL)
-            dbus_set_g_error (error, &dbus_error);
-
-          dbus_error_free (&dbus_error);
-        }
-      else if (error != NULL)
+      if (error != NULL)
         {
           g_set_error (error, DBUS_GERROR, DBUS_GERROR_FAILED,
                        _("Another thumbnail cache service is already running"));
         }
 
-      g_mutex_unlock (service->mutex);
+      tumbler_mutex_unlock (service->mutex);
 
       return FALSE;
     }
   
-  /* everything's fine, install the cache type D-Bus info */
-  dbus_g_object_type_install_info (G_OBJECT_TYPE (service),
-                                   &dbus_glib_tumbler_cache_service_object_info);
-
-  /* register the cache instance as a handler of the cache interface */
-  dbus_g_connection_register_g_object (service->connection, 
-                                       "/org/freedesktop/thumbnails/Cache1",
-                                       G_OBJECT (service));
-
-  g_mutex_unlock (service->mutex);
+  tumbler_mutex_unlock (service->mutex);
 
   return TRUE;
 }
